<?php

namespace Litepci\Jkkll\Providers;

use Illuminate\Support\ServiceProvider;

class JkkllServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'jkkll');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'jkkll');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerJkkll();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerJkkll()
    {
        $this->app->bind('jkkll', function($app) {
            return new Jkkll($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Jkkll', 'Lavalite\Jkkll\Facades\Jkkll');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litepci.jkkll', function ($app) {
            return $this->app->make('Litepci\Jkkll\Jkkll');
        });

                // Bind Cart to repository
        $this->app->bind(
            'Litepci\Jkkll\Interfaces\CartRepositoryInterface',
            \Litepci\Jkkll\Repositories\Eloquent\CartRepository::class
        );

        $this->app->register(\Litepci\Jkkll\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Litepci\Jkkll\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and jkkll's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'litepci.jkkll'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeJkkll::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litepci.jkkll'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litepci/jkkll.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/jkkll')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/jkkll')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
